package java.time;

import java.util.Collections;
import java.util.Arrays;
import java.io.DataInput;
import java.io.IOException;
import java.io.DataOutput;
import java.io.InvalidObjectException;
import java.io.ObjectInputStream;
import java.time.temporal.TemporalQueries;
import java.time.chrono.Chronology;
import java.time.temporal.TemporalAccessor;
import java.time.temporal.Temporal;
import java.time.temporal.UnsupportedTemporalTypeException;
import java.util.regex.Matcher;
import java.time.format.DateTimeParseException;
import java.util.Iterator;
import java.time.temporal.ChronoUnit;
import java.util.Objects;
import java.time.chrono.IsoChronology;
import java.time.temporal.TemporalAmount;
import java.time.temporal.TemporalUnit;
import java.util.List;
import java.util.regex.Pattern;
import java.io.Serializable;
import java.time.chrono.ChronoPeriod;

public final class Period implements ChronoPeriod, Serializable
{
    public static final Period ZERO;
    private static final long serialVersionUID = -3587258372562876L;
    private static final Pattern PATTERN;
    private static final List<TemporalUnit> SUPPORTED_UNITS;
    private final int years;
    private final int months;
    private final int days;
    
    public static Period ofYears(int n) {
        return create(n, 0, 0);
    }
    
    public static Period ofMonths(int n) {
        return create(0, n, 0);
    }
    
    public static Period ofWeeks(int x) {
        return create(0, 0, Math.multiplyExact(x, 7));
    }
    
    public static Period ofDays(int n) {
        return create(0, 0, n);
    }
    
    public static Period of(int n, int n2, int n3) {
        return create(n, n2, n3);
    }
    
    public static Period from(TemporalAmount temporalAmount) {
        if ((temporalAmount instanceof Period)) {
            return (Period)temporalAmount;
        }
        if (temporalAmount instanceof ChronoPeriod && !IsoChronology.INSTANCE.equals(((ChronoPeriod)temporalAmount).getChronology())) {
            throw new DateTimeException("Period requires ISO chronology: " + temporalAmount);
        }
        Objects.requireNonNull(temporalAmount, "amount");
        int intExact = 0;
        int intExact2 = 0;
        int intExact3 = 0;
        for (TemporalUnit obj : temporalAmount.getUnits()) {
            long value = temporalAmount.get(obj);
            if (obj == ChronoUnit.YEARS) {
                intExact = Math.toIntExact(value);
            }
            else if (obj == ChronoUnit.MONTHS) {
                intExact2 = Math.toIntExact(value);
            }
            else if (obj == ChronoUnit.DAYS) {
                intExact3 = Math.toIntExact(value);
            }
            else {
                throw new DateTimeException("Unit must be Years, Months or Days, but was " + obj);
            }
        }
        return create(intExact, intExact2, intExact3);
    }
    
    public static Period parse(CharSequence parsedData) {
        Objects.requireNonNull(parsedData, "text");
        Matcher matcher = Period.PATTERN.matcher(parsedData);
        if (matcher.matches()) {
            int n = "-".equals(matcher.group(1)) ? -1 : 1;
            String group = matcher.group(2);
            String group2 = matcher.group(3);
            String group3 = matcher.group(4);
            String group4 = matcher.group(5);
            if (group == null && group2 == null && group4 == null && group3 == null) {
                throw new DateTimeParseException("Text cannot be parsed to a Period", parsedData, 0);
            }
            try {
                return create(parseNumber(parsedData, group, n), parseNumber(parsedData, group2, n), Math.addExact(parseNumber(parsedData, group4, n), Math.multiplyExact(parseNumber(parsedData, group3, n), 7)));
            }
            catch (NumberFormatException cause) {
                throw new DateTimeParseException("Text cannot be parsed to a Period", parsedData, 0, cause);
            }
        }
        throw new DateTimeParseException("Text cannot be parsed to a Period", parsedData, 0);
    }
    
    private static int parseNumber(CharSequence parsedData, String s, int y) {
        if (s == null) {
            return 0;
        }
        int int1 = Integer.parseInt(s);
        try {
            return Math.multiplyExact(int1, y);
        }
        catch (ArithmeticException cause) {
            throw new DateTimeParseException("Text cannot be parsed to a Period", parsedData, 0, cause);
        }
    }
    
    public static Period between(LocalDate localDate, LocalDate localDate2) {
        return localDate.until(localDate2);
    }
    
    private static Period create(int n, int n2, int n3) {
        if ((n | n2 | n3) == 0x0) {
            return Period.ZERO;
        }
        return new Period(n, n2, n3);
    }
    
    private Period(int years, int months, int days) {
        this.years = years;
        this.months = months;
        this.days = days;
    }
    
    @Override
    public long get(TemporalUnit obj) {
        if (obj == ChronoUnit.YEARS) {
            return (long)this.getYears();
        }
        if (obj == ChronoUnit.MONTHS) {
            return (long)this.getMonths();
        }
        if (obj == ChronoUnit.DAYS) {
            return (long)this.getDays();
        }
        throw new UnsupportedTemporalTypeException("Unsupported unit: " + obj);
    }
    
    @Override
    public List<TemporalUnit> getUnits() {
        return Period.SUPPORTED_UNITS;
    }
    
    @Override
    public IsoChronology getChronology() {
        return IsoChronology.INSTANCE;
    }
    
    @Override
    public boolean isZero() {
        return this == Period.ZERO;
    }
    
    @Override
    public boolean isNegative() {
        return this.years < 0 || this.months < 0 || this.days < 0;
    }
    
    public int getYears() {
        return this.years;
    }
    
    public int getMonths() {
        return this.months;
    }
    
    public int getDays() {
        return this.days;
    }
    
    public Period withYears(int n) {
        if (n == this.years) {
            return this;
        }
        return create(n, this.months, this.days);
    }
    
    public Period withMonths(int n) {
        if (n == this.months) {
            return this;
        }
        return create(this.years, n, this.days);
    }
    
    public Period withDays(int n) {
        if (n == this.days) {
            return this;
        }
        return create(this.years, this.months, n);
    }
    
    @Override
    public Period plus(TemporalAmount temporalAmount) {
        Period from = from(temporalAmount);
        return create(Math.addExact(this.years, from.years), Math.addExact(this.months, from.months), Math.addExact(this.days, from.days));
    }
    
    public Period plusYears(long y) {
        if (y == 0L) {
            return this;
        }
        return create(Math.toIntExact(Math.addExact((long)this.years, y)), this.months, this.days);
    }
    
    public Period plusMonths(long y) {
        if (y == 0L) {
            return this;
        }
        return create(this.years, Math.toIntExact(Math.addExact((long)this.months, y)), this.days);
    }
    
    public Period plusDays(long y) {
        if (y == 0L) {
            return this;
        }
        return create(this.years, this.months, Math.toIntExact(Math.addExact((long)this.days, y)));
    }
    
    @Override
    public Period minus(TemporalAmount temporalAmount) {
        Period from = from(temporalAmount);
        return create(Math.subtractExact(this.years, from.years), Math.subtractExact(this.months, from.months), Math.subtractExact(this.days, from.days));
    }
    
    public Period minusYears(long n) {
        return (n == Long.MIN_VALUE) ? this.plusYears(Long.MAX_VALUE).plusYears(1L) : this.plusYears(-n);
    }
    
    public Period minusMonths(long n) {
        return (n == Long.MIN_VALUE) ? this.plusMonths(Long.MAX_VALUE).plusMonths(1L) : this.plusMonths(-n);
    }
    
    public Period minusDays(long n) {
        return (n == Long.MIN_VALUE) ? this.plusDays(Long.MAX_VALUE).plusDays(1L) : this.plusDays(-n);
    }
    
    @Override
    public Period multipliedBy(int y) {
        if (this == Period.ZERO || y == 1) {
            return this;
        }
        return create(Math.multiplyExact(this.years, y), Math.multiplyExact(this.months, y), Math.multiplyExact(this.days, y));
    }
    
    @Override
    public Period negated() {
        return this.multipliedBy(-1);
    }
    
    @Override
    public Period normalized() {
        long totalMonths = this.toTotalMonths();
        long value = totalMonths / 12L;
        int n = (int)(totalMonths % 12L);
        if (value == (long)this.years && n == this.months) {
            return this;
        }
        return create(Math.toIntExact(value), n, this.days);
    }
    
    public long toTotalMonths() {
        return (long)this.years * 12L + (long)this.months;
    }
    
    @Override
    public Temporal addTo(Temporal temporal) {
        this.validateChrono(temporal);
        if (this.months == 0) {
            if (this.years != 0) {
                temporal = temporal.plus((long)this.years, ChronoUnit.YEARS);
            }
        }
        else {
            long totalMonths = this.toTotalMonths();
            if (totalMonths != 0L) {
                temporal = temporal.plus(totalMonths, ChronoUnit.MONTHS);
            }
        }
        if (this.days != 0) {
            temporal = temporal.plus((long)this.days, ChronoUnit.DAYS);
            return temporal;
        }
        return temporal;
    }
    
    @Override
    public Temporal subtractFrom(Temporal temporal) {
        this.validateChrono(temporal);
        if (this.months == 0) {
            if (this.years != 0) {
                temporal = temporal.minus((long)this.years, ChronoUnit.YEARS);
            }
        }
        else {
            long totalMonths = this.toTotalMonths();
            if (totalMonths != 0L) {
                temporal = temporal.minus(totalMonths, ChronoUnit.MONTHS);
            }
        }
        if (this.days != 0) {
            temporal = temporal.minus((long)this.days, ChronoUnit.DAYS);
            return temporal;
        }
        return temporal;
    }
    
    private void validateChrono(TemporalAccessor obj) {
        Objects.requireNonNull(obj, "temporal");
        Chronology obj2 = (Chronology)obj.query(TemporalQueries.chronology());
        if (obj2 != null && !IsoChronology.INSTANCE.equals(obj2)) {
            throw new DateTimeException("Chronology mismatch, expected: ISO, actual: " + obj2.getId());
        }
        return;
    }
    
    @Override
    public boolean equals(Object o) {
        if (this == o) {
            return true;
        }
        if ((o instanceof Period)) {
            Period period = (Period)o;
            return this.years == period.years && this.months == period.months && this.days == period.days;
        }
        return false;
    }
    
    @Override
    public int hashCode() {
        return this.years + Integer.rotateLeft(this.months, 8) + Integer.rotateLeft(this.days, 16);
    }
    
    @Override
    public String toString() {
        if (this == Period.ZERO) {
            return "P0D";
        }
        StringBuilder sb = new StringBuilder();
        sb.append('P');
        if (this.years != 0) {
            sb.append(this.years).append('Y');
        }
        if (this.months != 0) {
            sb.append(this.months).append('M');
        }
        if (this.days != 0) {
            sb.append(this.days).append('D');
        }
        return sb.toString();
    }
    
    private Object writeReplace() {
        return new Ser((byte)14, this);
    }
    
    private void readObject(ObjectInputStream objectInputStream) throws InvalidObjectException {
        throw new InvalidObjectException("Deserialization via serialization delegate");
    }
    
    void writeExternal(DataOutput dataOutput) throws IOException {
        dataOutput.writeInt(this.years);
        dataOutput.writeInt(this.months);
        dataOutput.writeInt(this.days);
    }
    
    static Period readExternal(DataInput dataInput) throws IOException {
        return of(dataInput.readInt(), dataInput.readInt(), dataInput.readInt());
    }
    
    static {
        ZERO = new Period(0, 0, 0);
        PATTERN = Pattern.compile("([-+]?)P(?:([-+]?[0-9]+)Y)?(?:([-+]?[0-9]+)M)?(?:([-+]?[0-9]+)W)?(?:([-+]?[0-9]+)D)?", 2);
        SUPPORTED_UNITS = Collections.unmodifiableList(Arrays.asList(new TemporalUnit[] { ChronoUnit.YEARS, ChronoUnit.MONTHS, ChronoUnit.DAYS }));
    }
}
